package be.iminds.ilabt.jfed.ui.javafx.choosers;

import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.ui.javafx.am_list_gui.AMList;
import com.cathive.fx.guice.FXMLComponent;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.layout.VBox;
import javafx.util.StringConverter;

import javax.inject.Inject;
import javax.inject.Provider;

/**
 * AuthorityChooser
 */
@FXMLComponent
public class AuthorityChooser extends VBox {
    @FXML
    private ComboBox<AuthorityInfo> comboBox;
    @FXML
    private Button editButton;


    private final AuthorityList authorityList;
    private final Provider<AMList> amListProvider;

    @Inject
    public AuthorityChooser(AuthorityList authorityList, Provider<AMList> amListProvider) {
        this.authorityList = authorityList;
        this.amListProvider = amListProvider;
    }

    @FXML
    private void initialize() {

        comboBox.setConverter(new StringConverter<AuthorityInfo>() {
            @Override
            public String toString(AuthorityInfo authorityInfo) {
                assert authorityInfo != null;
                return authorityInfo.getSfaAuthority().getName();
            }

            @Override
            public AuthorityInfo fromString(String s) {
                throw new RuntimeException("unused");
            }
        });

        editable.addListener((observableValue, oldBoolean, newBoolean) -> comboBox.disableProperty().set(!newBoolean));

        showEditListButton.addListener((observableValue, oldBoolean, newBoolean) -> {
            editButton.setVisible(newBoolean);
            editButton.setManaged(newBoolean);
        });

        assert (authorityList != null);
        comboBox.setItems(authorityList.authorityInfosProperty());
        comboBox.getSelectionModel().selectFirst();
    }


    public AuthorityInfo getSelectedAuthority() {
        return comboBox.getSelectionModel().getSelectedItem();
    }

    public ReadOnlyObjectProperty<AuthorityInfo> selectedAuthorityProperty() {
        return comboBox.getSelectionModel().selectedItemProperty();
    }

    public void select(AuthorityInfo authorityInfo) {
        assert authorityInfo != null : "must always select something";
        comboBox.getSelectionModel().select(authorityInfo);
    }


    public void select(SfaAuthority authorityInfo) {
        assert authorityInfo != null : "must always select something";
        AuthorityInfo ai = authorityList.get(authorityInfo);
        select(ai);
    }

    @FXML
    public void editList() {
        amListProvider.get().show();
    }

    private BooleanProperty editable = new SimpleBooleanProperty(true);

    public BooleanProperty editableProperty() {
        return editable;
    }

    private BooleanProperty showEditListButton = new SimpleBooleanProperty(true);

    public BooleanProperty showEditListButtonProperty() {
        return showEditListButton;
    }

    private ObservableList<AuthorityInfo> filteredAuthorities = FXCollections.observableArrayList();
    private AuthorityFilter authorityFilter;
    private ListChangeListener<AuthorityInfo> listChangeListener;

    public void setAuthorityFilter(final AuthorityFilter authorityFilter) {
        assert authorityList.authorityInfosProperty() != null;

        if (authorityFilter == null) {
            this.authorityFilter = null;
            comboBox.setItems(authorityList.authorityInfosProperty());
            comboBox.getSelectionModel().selectFirst();
            return;
        }
        this.authorityFilter = authorityFilter;

        filteredAuthorities.clear();
        for (AuthorityInfo ai : authorityList.authorityInfosProperty()) {
            if (authorityFilter.includeAuthority(ai))
                filteredAuthorities.add(ai);
        }
        comboBox.setItems(filteredAuthorities);

        //init and set listener only once. It keeps listening, but it doesn't do anything when there is no filter
        if (listChangeListener != null) {
            listChangeListener = change -> {
                if (AuthorityChooser.this.authorityFilter == null)
                    return;

                while (change.next()) {
                    if (change.wasAdded()) {
                        for (AuthorityInfo a : change.getAddedSubList())
                            if (AuthorityChooser.this.authorityFilter.includeAuthority(a))
                                filteredAuthorities.add(a);
                    }
                    if (change.wasRemoved())
                        filteredAuthorities.removeAll(change.getRemoved());
                }
            };
            authorityList.authorityInfosProperty().addListener(listChangeListener);
        }
        comboBox.getSelectionModel().selectFirst();
    }

    public static interface AuthorityFilter {
        public boolean includeAuthority(AuthorityInfo ai);
    }
}
