package be.iminds.ilabt.jfed.experimenter_gui;

import javafx.application.Application;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

/**
 * User: twalcari
 * Date: 3/3/2015
 * Time: 11:06
 */
public class StartupParameters {
    private static final Logger LOG = LoggerFactory.getLogger(StartupParameters.class);

    public static final String JFED_URI_SCHEME = "jfed";

    public static final String PARAM_LOGIN_CERTIFICATE = "login-certificate";
    public static final String PARAM_LOGIN_PASSWORD = "password";
    public static final String PARAM_LOGIN_CERTIFICATE_STRING = "login-certificate-string";
    public static final String PARAM_SLICE_URN = "slice_urn";
    public static final String PARAM_RSPEC = "rspec";
    /**
     * Duration is defined in minutes. Values smaller than 2 hours will be ignored.
     */
    public static final String PARAM_SLICE_DURATION = "slice_duration";

    private final Map<String, String> namedParameters;

    public StartupParameters(Application.Parameters parameters) {
        if (!parameters.getNamed().isEmpty()) {
            this.namedParameters = parameters.getNamed();
        } else {
            this.namedParameters = new HashMap<>();

            for (String param : parameters.getRaw()) {
                try {
                    URI uri = new URI(param);

                    if (JFED_URI_SCHEME.equalsIgnoreCase(uri.getScheme())) {
                        LOG.debug("Found a jFed-URI with the following contents: {}", uri.getSchemeSpecificPart());

                        String[] nameValuePairs = uri.getSchemeSpecificPart().split("&");

                        for (String nameValuePair : nameValuePairs) {
                            int splitIndex = nameValuePair.indexOf("=");

                            if (splitIndex >= 0) {
                                String name = nameValuePair.substring(0, splitIndex);
                                String value = nameValuePair.substring(splitIndex + 1);

                                namedParameters.put(name, value);
                            } else {
                                LOG.warn("Could not parse parameter '{}'. Skipping.", nameValuePair);
                            }
                        }

                        break;    //only parse the first URI we encounter
                    } else {
                        LOG.warn("Encountered an URI, but with an unexpected scheme '{}'. Ignoring", uri.getScheme());
                    }
                } catch (URISyntaxException ignored) {
                    //this parameter is not an URI, we ignore it.
                }
            }
        }
    }

    public boolean contains(String parameterName) {
        return namedParameters.containsKey(parameterName);
    }

    public String getValue(String parameterName) {
        return namedParameters.get(parameterName);
    }
}
