package be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.controller.controller.ControllerController;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.TimeTextField;
import be.iminds.ilabt.jfed.rspec.model.controller.ExperimentCommand;
import be.iminds.ilabt.jfed.rspec.model.controller.Timestamp;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;
import org.controlsfx.validation.ValidationMessage;
import org.controlsfx.validation.ValidationSupport;
import org.controlsfx.validation.Validator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.stream.Collectors;

public class CommandPropertiesDialog extends GridPane {

    private static final Logger LOG = LoggerFactory.getLogger(CommandPropertiesDialog.class);

    private static final String NODE_PROPERTIES_FXML = "CommandPropertiesDialog.fxml";
    private final ControllerController controller;
    private final ExperimentCommand command;
    @FXML
    private TextField commandTextField;
    @FXML
    private TimeTextField startTextField;
    @FXML
    private Button okButton;
    @FXML
    private TextField tagTextField;


    private final Dialog dialog;
    private final DialogAction okAction;

    private final ValidationSupport validationSupport;

    private CommandPropertiesDialog(Window parent, ControllerController controller, final ExperimentCommand command) {
        this.controller = controller;
        this.command = command;

        //initialize the Scene
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(NODE_PROPERTIES_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        tagTextField.setText(command.getTag());
        commandTextField.setText(command.getCommand());
        startTextField.setTime(command.getStartingTime().toString());

        //initialize dialog
        dialog = new Dialog(parent, String.format("Properties of %s", command.getTag()));
        dialog.setMasthead(dialog.getTitle());
        dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.PENCIL, Color.BLACK));
        dialog.setContent(this);

        okAction = new DialogAction("OK", ButtonBar.ButtonType.OK_DONE,false,false, true, this::onOKAction);


        dialog.getActions().setAll(okAction, Dialog.ACTION_CANCEL);

        //initialize validation support
        validationSupport = new ValidationSupport();
        validationSupport.registerValidator(tagTextField, Validator.createEmptyValidator("Tag is a required field!"));
        validationSupport.registerValidator(commandTextField, Validator.createEmptyValidator("Command is a required field!"));
    }

    private void onOKAction(ActionEvent actionEvent) {
        if (validationSupport.isInvalid()) {
            JFDialogs.create()
                    .message(String.format("Please correct the following issue(s): %s",
                    validationSupport.getValidationResult().getMessages().stream()
                            .map(ValidationMessage::getText)
                            .collect(Collectors.joining(", "))))
                    .showWarning();
            return;
        }


        Timestamp newDuration;
        if (!command.getCommand().equals(commandTextField.getText())) {
            newDuration = new Timestamp(0);
        } else {
            newDuration = command.getDuration();
        }

        controller.updateCommand(command,
                new ExperimentCommand(tagTextField.getText(),
                        command.getNodeUniqueID(),
                        command.getBarrierSegmentOrderNumber(),
                        commandTextField.getText(),
                        new Timestamp(startTextField.toMillis()), newDuration));

        dialog.setResult(okAction);
        dialog.hide();


        Stage stage = (Stage) getScene().getWindow();
        stage.close();
    }

    public static void showSimplePropertiesDialog(Window parent, ControllerController controller, final ExperimentCommand command) {
        final CommandPropertiesDialog npd = new CommandPropertiesDialog(parent, controller, command);
        npd.showDialog();
    }

    private void showDialog() {
        dialog.show();
    }
}
