package be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.controller.controller.ControllerController;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.TimeTextField;
import be.iminds.ilabt.jfed.rspec.model.controller.ExperimentBarrierSegment;
import be.iminds.ilabt.jfed.rspec.model.controller.Timestamp;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class FastForwardDialog extends GridPane {
    private static final Logger LOG = LoggerFactory.getLogger(AddCommandDialog.class);

    private static final String FAST_FORWARD_FXML = "FastForwardDialog.fxml";
    private final ControllerController controller;
    @FXML
    private TimeTextField fastForwardTextField;
    @FXML
    private VBox radioContainer;

    private final Map<RadioButton, ExperimentBarrierSegment> segments = new HashMap<>();
    private final ToggleGroup group = new ToggleGroup();

    private final org.controlsfx.dialog.Dialog dialog;
    private final DialogAction fastforwardAction;


    private FastForwardDialog(ControllerController controller, Timestamp startingTime) {
        this.controller = controller;

        //initialize the Scene
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(FAST_FORWARD_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }


        fastForwardTextField.setTime(startingTime.toString());
        setOnKeyPressed(keyEvent -> {
            if (keyEvent.getCode() == KeyCode.ENTER) {
                onOKAction();
                keyEvent.consume();
            }
        });

        for (ExperimentBarrierSegment b : controller.getModel().getCommandModelRspec().getBarrierSegments()) {
            RadioButton rb = new RadioButton(b.getTag());
            rb.setToggleGroup(group);
            radioContainer.getChildren().add(rb);
            segments.put(rb, b);
        }
        group.selectToggle((RadioButton) radioContainer.getChildren().get(0));

        //initialize dialog
        dialog = new Dialog(null, "Fast Forward");
        dialog.setMasthead("Jump to a specific time of the experiment");
        dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.CLOCK_ALT, Color.BLACK));
        dialog.setContent(this);

        fastforwardAction =
                new DialogAction("OK", ButtonBar.ButtonType.OK_DONE, false, false, true, event -> onOKAction());

        dialog.getActions().setAll(fastforwardAction, Dialog.ACTION_CANCEL);

    }


    private void onOKAction() {
        controller.setElapsedTime(fastForwardTextField.toMillis(), segments.get(group.getSelectedToggle()));

        dialog.setResult(fastforwardAction);
        dialog.hide();
    }

    public static void showSimpleFastForwardDialog(ControllerController controller, Timestamp startingTime) {
        FastForwardDialog dialog = new FastForwardDialog(controller, startingTime);
        dialog.showDialog();

    }

    private void showDialog() {
        dialog.show();
    }

}
