package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.StartupParameters;
import be.iminds.ilabt.jfed.experimenter_gui.call_gui.TasksWindow;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewControllerFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.collections.ObservableMap;
import javafx.stage.Window;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 12/2/2014
 * Time: 16:37
 */
@Singleton
public class DialogsFactoryImpl implements DialogsFactory {

    private final TasksWindow tasksWindow;

    private final GeniUserProvider geniUserProvider;
    private final EasyModel easyModel;
    private final HighLevelController hlc;
    private final ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers;
    private final ExperimentViewControllerFactory experimentViewControllerFactory;
    private final ExperimentTasksFactory experimentTasksFactory;
    private final JFedGuiPreferences jFedPreferences;
    private final AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory
            automaticAggregateManagerWrapperFactory;
    private final TasksFactory tasksFactory;
    private final TaskService taskService;
    private final AuthorityListModel authorityListModel;
    private final AuthorityFinder authorityFinder;
    private final StartupParameters startupParameters;

    @Inject
    public DialogsFactoryImpl(TasksWindow tasksWindow, GeniUserProvider geniUserProvider, EasyModel easyModel, HighLevelController hlc,
                              ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers, ExperimentViewControllerFactory experimentViewControllerFactory,
                              ExperimentTasksFactory experimentTasksFactory, JFedGuiPreferences jFedPreferences,
                              AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory,
                              TasksFactory tasksFactory, TaskService taskService,
                              AuthorityListModel authorityListModel, AuthorityFinder authorityFinder, StartupParameters startupParameters) {
        this.tasksWindow = tasksWindow;
        this.geniUserProvider = geniUserProvider;
        this.easyModel = easyModel;
        this.hlc = hlc;
        this.experimentViewControllers = experimentViewControllers;
        this.experimentViewControllerFactory = experimentViewControllerFactory;
        this.experimentTasksFactory = experimentTasksFactory;
        this.jFedPreferences = jFedPreferences;
        this.automaticAggregateManagerWrapperFactory = automaticAggregateManagerWrapperFactory;
        this.tasksFactory = tasksFactory;
        this.taskService = taskService;
        this.authorityListModel = authorityListModel;
        this.authorityFinder = authorityFinder;
        this.startupParameters = startupParameters;
    }

    @Override
    public void showCreateSliceDialog(RequestRspecSource requestRspecSource) {
        new CreateSliceDialog(requestRspecSource, hlc, geniUserProvider, easyModel, experimentViewControllers, experimentViewControllerFactory, authorityListModel, authorityFinder, startupParameters).showDialog();
    }

    @Override
    public void showRebootDialog(Window parentWindow, ExperimentController experimentController) {
        new RebootDialog(parentWindow, experimentController, experimentTasksFactory, jFedPreferences, automaticAggregateManagerWrapperFactory).showDialog();
    }

    @Override
    public void showEditSshKeysDialog(Window parentWindow, ExperimentController experimentController) {
        new EditSshKeysDialog(parentWindow, experimentController, experimentTasksFactory, geniUserProvider, easyModel, jFedPreferences, automaticAggregateManagerWrapperFactory).showDialog();
    }

    @Override
    public void showShareSliceDialog(Window parentWindow, ExperimentController experimentController) {
        new ShareSliceDialog(parentWindow, experimentController, experimentTasksFactory, geniUserProvider, hlc, tasksWindow).showDialog();

    }

    @Override
    public void showCreateReservationDialog(Window parentWindow, RequestRspecSource requestRspecSource) {
        new CreateReservationDialog(parentWindow, requestRspecSource, hlc, geniUserProvider, easyModel, experimentViewControllers, experimentViewControllerFactory).showDialog();
    }

    @Override
    public void showRenewSliceDialog(Window parentWindow, ExperimentController experimentController) {
        new RenewSliceDialog(parentWindow, experimentController, experimentTasksFactory).showDialog();
    }

}
