package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.tasks.RecoverSliceTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.util.Callback;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;
import java.io.IOException;

/**
 * User: twalcari
 * Date: 12/5/13
 * Time: 9:21 PM
 */
public class RecoverSliceDialog extends VBox {

    private static final String RECOVER_SLICE_DIALOG_FXML = "RecoverSliceDialog.fxml";

    @FXML
    private TableView<Slice> slicesTableView;

    @FXML
    private TableColumn<Slice, String> sliceNameTableColumn;
    @FXML
    private TableColumn<Slice, String> subAuthTableColumn;
    //@FXML
    //private TableColumn<Slice, Date> expirationTableColumn;
    @FXML
    private HBox initializingBox;

    @FXML
    private ProgressIndicator refreshProgressIndicator;
    @FXML
    private Label refreshLabel;

    private final EasyModel easyModel;
    private final HighLevelController hlc;
    private final TasksFactory tasksFactory;

    private final EventHandler<MouseEvent> mouseEventEventHandler = mouseEvent -> {
        if (mouseEvent.getClickCount() == 2) {
            recoverExperiment();
        }

    };

    private final Dialog dialog;

    private final TaskService taskService;
    private final Provider<Stage> parentStageProvider;

    @Inject
    RecoverSliceDialog(EasyModel easyModel, HighLevelController hlc,  TasksFactory tasksFactory, TaskService taskService, @Named("parentStage") Provider<Stage> parentStageProvider) {
        this.easyModel = easyModel;
        this.hlc = hlc;
        this.tasksFactory = tasksFactory;
        this.taskService = taskService;
        this.parentStageProvider = parentStageProvider;

        //initialize the Scene
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(RECOVER_SLICE_DIALOG_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }


        slicesTableView.itemsProperty().bind(this.easyModel.slicesProperty());
        sliceNameTableColumn.prefWidthProperty().bind(slicesTableView.widthProperty().multiply(0.6).subtract(1));
        subAuthTableColumn.prefWidthProperty().bind(slicesTableView.widthProperty().multiply(0.4).subtract(1));
        // expirationTableColumn.prefWidthProperty().bind(slicesTableView.widthProperty().multiply(0.25).subtract(1));

        sliceNameTableColumn.setCellValueFactory(new PropertyValueFactory<>("name"));
        sliceNameTableColumn.setCellFactory(new Callback<TableColumn<Slice, String>, TableCell<Slice, String>>() {
            @Override
            public TableCell<Slice, String> call(TableColumn<Slice, String> sliceStringTableColumn) {
                return new TableCell<Slice, String>() {
                    {
                        setOnMouseClicked(mouseEventEventHandler);
                    }

                    @Override
                    protected void updateItem(String s, boolean b) {
                        if (s != null) {
                            setText(s);
                        } else {
                            setText("");
                        }
                    }
                };
            }
        });
        subAuthTableColumn.setCellValueFactory(t -> new SimpleStringProperty(t.getValue().getSubAuthorityName()));
        subAuthTableColumn.setCellFactory(new Callback<TableColumn<Slice, String>, TableCell<Slice, String>>() {
            @Override
            public TableCell<Slice, String> call(TableColumn<Slice, String> sliceStringTableColumn) {
                return new TableCell<Slice, String>() {
                    {
                        setOnMouseClicked(mouseEventEventHandler);
                    }

                    @Override
                    protected void updateItem(String s, boolean b) {
                        if (s != null) {
                            setText(s);
                        } else {
                            setText("");
                        }
                    }
                };
            }
        });

        //sort by default on slice name
        sliceNameTableColumn.setComparator(String.CASE_INSENSITIVE_ORDER);
        slicesTableView.getSortOrder().add(sliceNameTableColumn);

        //show loading progressview when initializing
        javafx.concurrent.Task loadingTask = new javafx.concurrent.Task<Void>() {
            Task resolveTask = hlc.resolveUserAndSlices();

            @Override
            protected Void call() throws Exception {
                while (resolveTask.isActive()) {
                    Thread.sleep(100);
                }

                //check if some resolveSlice-calls are still running
                for (Slice slice : RecoverSliceDialog.this.easyModel.getSlices()) {
                    while (hlc.getAggregatesForSlice(slice).isActive())
                        Thread.sleep(100);
                }


                return null;
            }

            @Override
            protected void succeeded() {
                initializingBox.setVisible(false);
            }
        };

        final Thread thread = new Thread(loadingTask);
        thread.setDaemon(true);
        thread.setName("Recover-Loading-Task");
        thread.start();

        dialog = new Dialog(parentStageProvider.get(), "Recover an existing experiment", false /* lightweight */);

        dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.AMBULANCE, Color.GREEN));
        dialog.setMasthead("Select the experiment to recover");
        dialog.setContent(this);

        Action refreshAction = new DialogAction("Refresh", ButtonBar.ButtonType.OTHER,
                false, false, false, actionEvent -> refreshSlicesList());
        refreshAction.setGraphic(GlyphUtils.createSmallGlyph(FontAwesome.Glyph.REFRESH, Color.GREEN).useGradientEffect());

        Action startExperimentAction =
                new DialogAction("Recover Experiment", ButtonBar.ButtonType.OK_DONE, false, false, true,
                        actionEvent -> recoverExperiment());
        dialog.getActions().setAll(refreshAction, startExperimentAction, Dialog.ACTION_CANCEL);

    }


    public void showDialog() {
        dialog.show();
    }

    private void refreshSlicesList() {
        refreshProgressIndicator.setVisible(true);
        refreshLabel.setText("");
        final HighLevelController.ListSlicesTask refreshSlicesTask = hlc.getSlicesForUser();

        refreshSlicesTask.addCallback((task, taskExecution, state) -> Platform.runLater(() -> {
            refreshProgressIndicator.setVisible(false);
            refreshLabel.setText(String.format("Received %d experiments",
                    refreshSlicesTask.getSlices().size()));
        }));
        hlc.submitTask(refreshSlicesTask);


    }

    public void recoverExperiment() {

        if (slicesTableView.getSelectionModel().isEmpty()) {
            JFDialogs.create()
                    .owner(this)
                    .message("Please select the slice you wish to recover")
                    .masthead("No slice selected")
                    .showError();
            return;
        }
        dialog.hide();

        final Slice recoverSlice = slicesTableView.getSelectionModel().getSelectedItem();

        RecoverSliceTask recoverSliceTask = tasksFactory.createRecoverSliceTask(recoverSlice);
        taskService.submitTask(recoverSliceTask);


    }
}
