package be.iminds.ilabt.jfed.experimenter_gui.editor;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasFactory;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.EditableExperimentCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerNodeFactory;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerUI;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs.AddMultipleCommandDialog;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.DialogsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.bo.NodeDescription;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.ChannelToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.NodeDescriptionToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.raw.RawRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecCheckReason;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationEngine;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationResult;
import be.iminds.ilabt.jfed.experimenter_gui.slice.NodePropertiesDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.util.FileUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.ImageUtil;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.StringRspec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.ProxyPreferencesManager;
import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.ButtonType;
import javafx.scene.input.MouseDragEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.TilePane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import org.controlsfx.dialog.CommandLinksDialog;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import javax.inject.Inject;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 11/5/13
 * Time: 6:24 PM
 */
public class ExperimentEditor {


    public static final String RSPEC_EXTENSION = ".rspec";
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentEditor.class);

    private final DialogsFactory dialogsFactory;
    private final JFedConfiguration conf;
    private final AuthorityList authorityList;

    private final NodePropertiesDialogFactory nodePropertiesDialog;
    private final ImageUtil imageUtil;
    private final GeniUserProvider geniUserProvider;
    private final ProxyPreferencesManager proxyPreferencesManager;
    private final ControllerNodeFactory controllerNodeFactory;

    private final CanvasFactory canvasFactory;
    private final JFedGuiPreferences jFedPreferences;
    private final StringProperty statusProperty = new SimpleStringProperty();
    private ToolboxItem draggedItem = null;
    private final RspecValidationEngine validationEngine = new RspecValidationEngine();
    private final EventHandler<MouseEvent> toolboxOnDragDetectedHandler = mouseEvent -> {
        //set selected toolboxitem as draggedItem
        ToolboxItem originalItem = (ToolboxItem) mouseEvent.getSource();
        // assert (originalItem instanceof NodeDescriptionToolboxItem);
        draggedItem = originalItem;

        //start dragging animation
        //setCursor(Cursor.CROSSHAIR);

        setStatus("Drop the item somewhere in the canvas.");

        //make JavaFX fire drag-events
        originalItem.startFullDrag();

        mouseEvent.consume();

    };

    private EditableExperimentCanvas canvas;
    private RawRspecEditor rawRspecEditor;
    private ControllerUI experimentController;
    @FXML
    private TilePane tpComputingElements;
    @FXML
    private VBox toolbox;
    @FXML
    private BorderPane root;
    //TODO find a way to get this handler to work
    private final EventHandler<MouseDragEvent> onMouseDragReleasedHandlerOutsideCanvas =
            mouseDragEvent -> {
                if (draggedItem != null) {
                    draggedItem = null;
                    setStatus("Item can only be dropped on the canvas");
                    root.setCursor(null);

                    mouseDragEvent.consume();
                }
            };

    private File file;
    private StringProperty name = new SimpleStringProperty();
    /**
     * this variable contains the current model(editor) when the canvas or controller is visible
     */
    @Nullable
    private ModelRspecEditor modelRspecEditor;

    /**
     * This starts the experiment editor. If the rspecSource can be succesfully parsed, it opens in graphical mode.
     * Otherwise, it opens in the RSpec-editor-view.
     *
     * @param dialogsFactory
     * @param conf
     * @param authorityList
     * @param nodePropertiesDialog
     * @param imageUtil
     * @param geniUserProvider
     * @param proxyPreferencesManager
     * @param controllerNodeFactory
     * @param canvasFactory
     */
    @Inject
    public ExperimentEditor(DialogsFactory dialogsFactory, JFedConfiguration conf,
                            AuthorityList authorityList, NodePropertiesDialogFactory nodePropertiesDialog,
                            ImageUtil imageUtil, GeniUserProvider geniUserProvider,
                            ProxyPreferencesManager proxyPreferencesManager, ControllerNodeFactory controllerNodeFactory,
                            CanvasFactory canvasFactory, JFedGuiPreferences jFedPreferences) {
        super();

        this.dialogsFactory = dialogsFactory;
        this.conf = conf;
        this.authorityList = authorityList;
        this.nodePropertiesDialog = nodePropertiesDialog;
        this.imageUtil = imageUtil;
        this.geniUserProvider = geniUserProvider;
        this.proxyPreferencesManager = proxyPreferencesManager;
        this.controllerNodeFactory = controllerNodeFactory;
        this.canvasFactory = canvasFactory;
        this.jFedPreferences = jFedPreferences;
    }

    public void setContent(RequestRspecSource rspecSource, String name, File file) {
        this.file = file;
        this.name.set(name);

        //initialize GUI
        FXModelRspec modelRspec = (FXModelRspec) rspecSource.getModelRspec(ModelRspecType.FX);
        if (modelRspec == null) {
            //there is an error in the RSpec preventing it from being converted to a model.
            // So we show the raw RSpec XML and continue
            modelRspecEditor = null;
            showRawRspecEditor(rspecSource.getStringRspec(), true);
        } else {
            modelRspecEditor = new ModelRspecEditor(modelRspec, authorityList);

            if (!rspecSource.isXmlBased()) {
                //rearrange nodes on circle if not all positions are defined
                if (!modelRspecEditor.areAllNodePositionsDefined()) {
                    modelRspecEditor.arrangeNodesOnCircle();
                }
                showEditableExperimentCanvas(modelRspecEditor);
            } else {
                showRawRspecEditor(rspecSource.getStringRspec(), true);
            }

            //show unbound nodes dialog if necessary
            Collection<FXRspecNode> unboundNodes = modelRspecEditor.getUnboundNodes();

            if (!unboundNodes.isEmpty()) {
                Platform.runLater(new FutureTask<>(new ProcessUnboundNodesTask(unboundNodes)));
            }
        }

    }


    @FXML
    private void initialize() {
        //initialize event handlers
        root.setOnMouseDragReleased(onMouseDragReleasedHandlerOutsideCanvas);

        //load toolbox content
        loadToolboxContent();
    }


    public RawRspecEditor getRawRspecEditor() {
        return rawRspecEditor;
    }

    public boolean isCanvasVisible() {
        return this.canvas != null;
    }

    public boolean isExperimentControllerVisible() {
        return this.experimentController != null;
    }

    public boolean isRawRspecEditorVisible() {
        return this.rawRspecEditor != null;
    }

    private void showEditableExperimentCanvas(ModelRspecEditor modelRspecEditor) {
        this.canvas = canvasFactory.createEditableExperimentCanvas(this, modelRspecEditor);
        root.setCenter(this.canvas);
        root.setLeft(toolbox);

        this.rawRspecEditor = null;

        if (this.experimentController != null) {
            experimentController.getController().unregister();
            this.experimentController = null;
        }

        modelRspecEditor.setEditableExperimentCanvas(canvas);
    }

    private void showRawRspecEditor(StringRspec stringRspec, boolean dirty) {
        this.rawRspecEditor = new RawRspecEditor(stringRspec);
        root.setLeft(null);
        root.setCenter(this.rawRspecEditor);

        if (this.canvas != null)
            this.canvas.unregisterModel();

        this.canvas = null;

        if (this.experimentController != null) {
            experimentController.getController().unregister();
            this.experimentController = null;
        }
    }

    private void showExperimentController(ModelRspecEditor modelRspecEditor) {
        this.experimentController =
                new ControllerUI(modelRspecEditor, nodePropertiesDialog, imageUtil, geniUserProvider, proxyPreferencesManager, controllerNodeFactory, jFedPreferences);

        root.setLeft(null);
        root.setCenter(this.experimentController);

        if (this.canvas != null)
            this.canvas.unregisterModel();

        this.canvas = null;
        modelRspecEditor.setEditableExperimentCanvas(null);
        this.rawRspecEditor = null;
    }

    public boolean switchToRawRspecEditor() {
        if (canvas == null && experimentController == null)
            return true;

        RequestRspecSource newRspecSource = getRequestRspecSource();

        assert (newRspecSource != null);

        showRawRspecEditor(newRspecSource.getStringRspec(), false);

        modelRspecEditor = null;

        return true;

    }

    public boolean switchToEditableExperimentCanvas(FXModelRspec fxModelRspec) {
        if (rawRspecEditor == null && experimentController == null)
            return true;

        if (rawRspecEditor != null) {
//            RequestRspecSource newRspecSource = getRequestRspecSource();

//            assert (newRspecSource != null);

            modelRspecEditor =
                    new ModelRspecEditor(fxModelRspec,/*(FXModelRspec) newRspecSource.getModelRspec(ModelRspecType.FX)*/authorityList);

            showEditableExperimentCanvas(modelRspecEditor);
        } else {

            //we're switching from the experimentController-UI
            assert (modelRspecEditor != null);
            showEditableExperimentCanvas(modelRspecEditor);
        }

        return true;
    }

    public boolean switchToExperimentController(FXModelRspec fxModelRspec) {
        if (rawRspecEditor == null && canvas == null)
            return true;

        if (rawRspecEditor != null) {
//            RequestRspecSource newRspecSource = getRequestRspecSource();

//            assert (newRspecSource != null);
            modelRspecEditor =
                    new ModelRspecEditor(fxModelRspec,/*(FXModelRspec) newRspecSource.getModelRspec(ModelRspecType.FX)*/authorityList);
            showExperimentController(modelRspecEditor);
        } else {
            showExperimentController(modelRspecEditor);
        }

        return true;
    }


    /**
     * For the moment, this function will initialize some hard-coded Rspec-nodes.
     * This should be loaded dynamically from the Rspec advertisments later on..
     */
    private void loadToolboxContent() {
        for (NodeDescription nd : conf.getNodeDescriptions()) {
            NodeDescriptionToolboxItem ndti = new NodeDescriptionToolboxItem(nd);
            registerToolboxDragHandlers(ndti);
            tpComputingElements.getChildren().add(ndti);
        }

        ChannelToolboxItem channelToolboxItem = new ChannelToolboxItem();
        registerToolboxDragHandlers(channelToolboxItem);
        tpComputingElements.getChildren().add(channelToolboxItem);
    }

    private void registerToolboxDragHandlers(final ToolboxItem toolboxItem) {
        toolboxItem.setOnDragDetected(toolboxOnDragDetectedHandler);
    }

    public void addCommand() {
        if (isExperimentControllerVisible())
            AddMultipleCommandDialog.showSimpleAddDialog(root.getScene().getWindow(), experimentController.getController(), experimentController.getModelRspec().getNodes());
    }

    public void addBarrier() {
        if (isExperimentControllerVisible())
            experimentController.getController().addBarrierSegment();
    }

    public boolean save() {
        FileChooser fc = new FileChooser();
        fc.setTitle("Save experiment");
        fc.getExtensionFilters().add(FileUtils.RSPEC_EXTENSION_FILTER);
        fc.setInitialFileName(name.get() + RSPEC_EXTENSION);


        //set the default save dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));

        if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST)) {
            File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST));
            if (dir.exists()) {
                fc.setInitialDirectory(dir);
            }
        }

        if (file != null) {
            fc.setInitialDirectory(file.getParentFile());
            fc.setInitialFileName(file.getName());
        }

        file = fc.showSaveDialog(root.getScene().getWindow());

        if (file == null)
            return false;


        //save folder in which the file was saved for future file selector dialogs
        jFedPreferences.setString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST,
                file.getParentFile().getAbsolutePath());

        if (!file.getAbsolutePath().endsWith(RSPEC_EXTENSION))
            file = new File(file.getAbsolutePath() + RSPEC_EXTENSION);


        setName(file.getName().substring(0, file.getName().length() - RSPEC_EXTENSION.length()));
        final File saveFile = file;
        final String fileContents = getRequestRspecSource().getRspecXmlString();
        final javafx.concurrent.Task<Void> saveTask = new javafx.concurrent.Task<Void>() {

            @Override
            protected Void call() throws Exception {
                updateMessage("Saving manifest in \"" + saveFile.getAbsolutePath() + "\"");

                try (FileWriter writer = new FileWriter(saveFile)) {
                    writer.write(fileContents);
                    writer.flush();
                    writer.close();
                } catch (final IOException ex) {
                    LOG.error("Error while saving experiment to file '" + file.getAbsolutePath() + "'", ex);
                    Platform.runLater(new Runnable() {
                        @Override
                        public void run() {
                            JFDialogs.create()
                                    .owner(this)
                                    .message("An error occurred while saving the rspec.")
                                    .masthead("Error")
                                    .title("Error")
                                    .showException(ex);

                        }
                    });
                }
                updateMessage("Successfully saved experiment in \"" + saveFile.getAbsolutePath() + "\"");

                return null;
            }
        };
        saveTask.messageProperty().addListener(observable -> setStatus(saveTask.getMessage()));

        new Thread(saveTask).start();
        return true;
    }

    public void setStatus(String status) {
        statusProperty.set(status);
    }

    public EditableExperimentCanvas getCanvas() {
        return canvas;
    }

    public ControllerUI getExperimentController() {
        return experimentController;
    }

    public RequestRspecSource getRequestRspecSource() {

        if (rawRspecEditor != null) {
            return new RequestRspecSource(rawRspecEditor.getResultRspec(), ModelRspecType.FX);

        } else {
            return new RequestRspecSource(modelRspecEditor.getModelRspec());
        }
    }

    public File getFile() {
        return file;
    }


    public StringProperty statusProperty() {
        return statusProperty;
    }

    public String getName() {
        return name.get();
    }

    public void setName(String name) {
        this.name.set(name);
    }

    public StringProperty nameProperty() {
        return name;
    }

    public ToolboxItem getDraggedItem() {
        return draggedItem;
    }

    public void clearDraggedItem() {
        draggedItem = null;
    }


    public void startExperiment() {
        RequestRspecSource requestRspecSource = prepareRequestRspecSource();
        if (requestRspecSource == null) return;


        if (requestRspecSource.getAllComponentManagerUrns().isEmpty()) {
            JFDialogs.create()
                    .owner(this)
                    .message("jFed could not find component managers in the RSpec XML. " +
                            "Either there is an error in the RSpec, or jFed does not support this RSpec. " +
                            "Either way, it is impossible to continue, \"Run Experiment\" aborted.")
                    .masthead("Fatal Error")
                    .showError();
            return;
        }

        Set<GeniUrn> unknownComponentManagerUrns = getUnkownComponentManagerUrns(requestRspecSource);

        if (!unknownComponentManagerUrns.isEmpty()) {
            String urns = unknownComponentManagerUrns.stream().map(GeniUrn::toString).collect(Collectors.joining(", "));

            boolean isStitching =
                    requestRspecSource.getStringRspec().isStitching(authorityList.getAuthorityListModel(), authorityList.finder());
            if (!isStitching) {
                JFDialogs.create()
                        .owner(root.getScene().getWindow())
                        .message("jFed does not have information about some component managers in the RSpec XML: " + urns + " \n" +
                                "It is impossible to correctly continue, \"Run Experiment\" aborted.")
                        .masthead("Fatal Error")
                        .showError();
                return;
            } else
                JFDialogs.create()
                        .owner(root.getScene().getWindow())
                        .message("jFed does not have information about some component managers in the RSpec XML: " + urns + " \n" +
                                "Since the RSpec uses stitching, it is possible to continue. " +
                                "There might be an error later on, or the problem might fix itself.")
                        .masthead("Dragons Ahead!")
                        .showWarning();

        }

        dialogsFactory.showCreateSliceDialog(requestRspecSource);
    }

    private Set<GeniUrn> getUnkownComponentManagerUrns(RequestRspecSource requestRspecSource) {
        Set<GeniUrn> unknownComponentManagerUrns = new HashSet<>();
        for (GeniUrn componentManagerUrn : requestRspecSource.getAllComponentManagerUrns()) {
            SfaAuthority auth = authorityList.finder().findByUrn(componentManagerUrn, AuthorityFinder.Purpose.CREATE_SLIVER);
            if (auth == null) {
                unknownComponentManagerUrns.add(componentManagerUrn);
            }
        }
        return unknownComponentManagerUrns;
    }


    public void createReservation() {

        RequestRspecSource requestRspecSource = prepareRequestRspecSource();

        if (requestRspecSource.getAllComponentManagerUrns().isEmpty()) {
            JFDialogs.create()
                    .owner(this)
                    .message("jFed could not find component managers in the RSpec XML. " +
                            "Either there is an error in the RSpec, or jFed does not support this RSpec. " +
                            "Either way, it is impossible to continue, \"Run Experiment\" aborted.")
                    .masthead("Fatal Error")
                    .showError();
            return;
        }
        Set<GeniUrn> unknownComponentManagerUrns = getUnkownComponentManagerUrns(requestRspecSource);
        if (!unknownComponentManagerUrns.isEmpty()) {
            String urns = unknownComponentManagerUrns.stream().map(GeniUrn::toString).collect(Collectors.joining(", "));

            JFDialogs.create()
                    .message("jFed does not have information about some component managers in the RSpec XML: " + urns + " \n" +
                            "It is impossible to correctly continue, \"Run Experiment\" aborted.")
                    .masthead("Fatal Error")
                    .showError();
            return;
        }

        //make sure the RSpec only contains reservable resources
        Set<SfaAuthority> unreservableComponentManagerUrns =
                requestRspecSource.getAllComponentManagerUrns().stream()
                        .map(urn -> authorityList.finder().findByUrn(urn, AuthorityFinder.Purpose.CREATE_SLIVER))
                        .filter(auth -> !(auth.hasSpecialCase(SfaAuthority.SpecialCase.SUPPORTS_RESERVATIONS_GENI)
                                || auth.hasSpecialCase(SfaAuthority.SpecialCase.SUPPORTS_RESERVATIONS_NITOS)))
                        .collect(Collectors.toSet());
        if (!unreservableComponentManagerUrns.isEmpty()) {

            JFDialogs.create()
                    .message("This RSpec contains resources that cannot be reserved in advance. Please remove all resources from the following authorities to continue: " +
                            unreservableComponentManagerUrns.stream().map(SfaAuthority::getHrn).collect(Collectors.joining(", ")))
                    .masthead("Unsupported authorities detected")
                    .showError();

            return;
        }


        dialogsFactory.showCreateReservationDialog(root.getScene().getWindow(), requestRspecSource);

    }

    private RequestRspecSource prepareRequestRspecSource() {
        RequestRspecSource requestRspecSource = getRequestRspecSource();

        RspecValidationResult verifyRspecResult =
                validationEngine.validateRSpec(RspecCheckReason.RUN_EXPERIMENT, requestRspecSource, root.getScene().getWindow());
        if (verifyRspecResult != null) {

            if (verifyRspecResult.isAbort()) {
                //user aborted (and is aware of that already so no need show dialog)
                return null;
            }

            requestRspecSource = verifyRspecResult.getReplacementRequestRspecSource();
        }


        if (rawRspecEditor != null) {
            rawRspecEditor.setContent(requestRspecSource.getRspecXmlString());
        } else {
            modelRspecEditor = new ModelRspecEditor((FXModelRspec) requestRspecSource.getModelRspec(ModelRspecType.FX), authorityList);
        }

        //create the modelRspecEditor if it doesn't exist yet.
        if (modelRspecEditor == null) {
            modelRspecEditor = new ModelRspecEditor((FXModelRspec) requestRspecSource.getModelRspec(ModelRspecType.FX), authorityList);
        }


        Collection<FXRspecNode> unboundNodes = modelRspecEditor.getUnboundNodes();

        if (!unboundNodes.isEmpty()) {
            ProcessUnboundNodesTask processUnboundNodesTask = new ProcessUnboundNodesTask(unboundNodes);

            if (!processUnboundNodesTask.call())
                return null;
        }
        return requestRspecSource;
    }


    public ModelRspecEditor getModelRspecEditor() {
        return modelRspecEditor;
    }


    private class ProcessUnboundNodesTask implements Callable<Boolean> {
        private final Collection<FXRspecNode> unboundNodes;

        private ProcessUnboundNodesTask(Collection<FXRspecNode> unboundNodes) {
            this.unboundNodes = unboundNodes;
        }


        @Override
        public Boolean call() {

            try {

                if (modelRspecEditor == null) {
                    JFDialogs.create()
                            .message("jFed detected unbound nodes. It can however not automatically resolve this situation, " +
                                    "as information would be loss in the process.")
                            .title("Unbound nodes detected")
                            .showError();

                }


                CommandLinksDialog.CommandLinksButtonType resolveNowButtonType =
                        new CommandLinksDialog.CommandLinksButtonType("Assign these nodes now",
                                "Let me choose the testbed to assign the unbound nodes to.", true);


                CommandLinksDialog.CommandLinksButtonType resolveLaterButtonType =
                        new CommandLinksDialog.CommandLinksButtonType("Resolve later",
                                "I want to resolve this issue manually, or want to edit the unbound RSpec", false);

                CommandLinksDialog dialog = new CommandLinksDialog(resolveNowButtonType, resolveLaterButtonType);
                dialog.initOwner(root.getScene().getWindow());
                dialog.setTitle("Unbound nodes detected");
                dialog.setHeaderText("Unbound nodes detected");
                dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.PUZZLE_PIECE, Color.CHOCOLATE));
                dialog.setContentText(String.format("This RSpec contains %d unbound nodes. " +
                                "These nodes need to be assigned to a testbed before starting the experiment.",
                        unboundNodes.size()));

                Optional<ButtonType> result = dialog.showAndWait();

                if (!result.isPresent() || result.get() != resolveNowButtonType.getButtonType())
                    return false;

                List<AuthorityInfo> bindableAuthorities = new ArrayList<>();

                for (JFedConfiguration.TestbedDescription testbedDescription : conf.getTestbedDescriptions()) {
                    if (testbedDescription.isShowAsBindable()) {
                        assert (authorityList.getByUrnExact(testbedDescription.getUrn()) != null) :
                                testbedDescription.getUrn() + " cannot be found in authorityList";

                        bindableAuthorities.add(authorityList.getByUrnExact(testbedDescription.getUrn()));
                    }
                }

                Collections.sort(bindableAuthorities, (o1, o2) -> o1.getName().compareTo(o2.getName()));

                Optional<AuthorityInfo> authorityInfo = JFDialogs.create()
                        .owner(ExperimentEditor.this)
                        .title("Unbound nodes detected")
                        .graphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.PUZZLE_PIECE, Color.CHOCOLATE))
                        .masthead(String.format("Please select the testbed to which the %d unbound nodes must be assigned:",
                                unboundNodes.size()))
                        .showChoices(bindableAuthorities);

                if (authorityInfo.isPresent()) {
                    modelRspecEditor.resolveUnboundRspecNodesToAuthority(authorityInfo.get().getSfaAuthority());

                    if (isRawRspecEditorVisible()) {
                        showRawRspecEditor(new RequestRspecSource(modelRspecEditor.getModelRspec()).getStringRspec(), false);
                    }

                    return true;
                }

                return false;
            } catch (Throwable ex) {
                LOG.error("Error in ProcessUnboundNodesTask", ex);
                throw ex;
            }
        }
    }

}
