package be.iminds.ilabt.jfed.experimenter_gui.editor;

import be.iminds.ilabt.jfed.experimenter_gui.ui.ExperimentEditorListProvider;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import com.cathive.fx.guice.GuiceFXMLLoader;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.io.File;

/**
 * User: twalcari
 * Date: 12/8/2014
 * Time: 15:58
 */
@Singleton
public class ExperimentEditorTabFactory {

    private final GuiceFXMLLoader loader;
    private final ExperimentEditorListProvider experimentEditorListProvider;

    @Inject
    public ExperimentEditorTabFactory(GuiceFXMLLoader loader, ExperimentEditorListProvider experimentEditorListProvider) {
        this.loader = loader;
        this.experimentEditorListProvider = experimentEditorListProvider;
    }


    public ExperimentEditorTab createExperimentEditorTab() {
        return new ExperimentEditorTab(new RequestRspecSource(new FXModelRspec("request")),
                getNextNewExperimentDefinitionName(), null, loader);
    }

    /**
     * Returns a new unique experiment definition name
     *
     * @return
     */
    private String getNextNewExperimentDefinitionName() {
        String baseName = "Untitled";

        if (!experimentDefinitionNameExists(baseName))
            return baseName;

        int counter = 1;
        String name = baseName + counter;
        while (experimentDefinitionNameExists(name)) {
            counter++;
            name = baseName + counter;
        }
        return name;
    }


    /**
     * Tests whether a given experiment definition is already open in the GUI
     *
     * @param name
     * @return
     */
    private boolean experimentDefinitionNameExists(String name) {
        return experimentEditorListProvider.getExperimentEditors()
                .stream()
                .map(ExperimentEditor::getName)
                .anyMatch(name::equals);
    }

    public ExperimentEditorTab createExperimentEditorTab(RequestRspecSource rspecSource, String name, File file) {
        return new ExperimentEditorTab(rspecSource, name, file, loader);
    }

}
