package be.iminds.ilabt.jfed.experimenter_gui.editor.properties;

import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

/**
 * User: twalcari
 * Date: 9/10/2014
 * Time: 09:44
 */
public class RoutableIPTab extends AdaptivePropertiesTab<FXRspecNode> {
    private static final Logger LOG = LoggerFactory.getLogger(RoutableIPTab.class);
    private static final String BOOT_SCRIPTS_FXML = "RoutableIP.fxml";

    private final AdaptivePropertiesDialog adaptivePropertiesDialog;

    @FXML private CheckBox routableIPCheckbox;


    public RoutableIPTab(AdaptivePropertiesDialog adaptivePropertiesDialog) {
        super("Routable Control IP");
        this.adaptivePropertiesDialog = adaptivePropertiesDialog;


        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(BOOT_SCRIPTS_FXML));
        fxmlLoader.setController(this);
        try {
            Node contents = fxmlLoader.load();
            setContent(contents);
        } catch (IOException e) {
            LOG.error("Error loading FXML: " + e.getMessage(), e);
            throw new RuntimeException("Error loading FXML: " + e.getMessage(), e);
        }
    }

    @Override
    public void load(FXRspecNode rspecNode) {
        routableIPCheckbox.setSelected(rspecNode.getRoutableControlIp());
    }

    @Override
    public void save(FXRspecNode rspecNode) {
        rspecNode.setRoutableControlIp(routableIPCheckbox.isSelected());
    }

    @Override
    public void onTabShown() {

    }

    @Override
    public boolean shouldBeVisible(FXRspecNode rspecNode) {
        final String currentSliverTypeName = adaptivePropertiesDialog.getCurrentSliverTypeName();
        return "emulab-xen".equals(currentSliverTypeName) ||
                "emulab-openvz".equals(currentSliverTypeName);
    }

    @Override
    public String validateConfiguration() {
        return null;
    }


//    public void onHelpLinkClicked() {
//        if(java.awt.Desktop.isDesktopSupported())
//        {
//            try {
//                java.awt.Desktop.getDesktop().browse(new URI("?"));
//            } catch (IOException | URISyntaxException e) {
//                LOG.warn("Exception starting browser", e);
//            }
//        }
//    }
}
