package be.iminds.ilabt.jfed.experimenter_gui.preferences.subsections;

import be.iminds.ilabt.jfed.experimenter_gui.preferences.AbstractPreferencesSubPane;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.ui.javafx.choosers.AuthorityChooser;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.beans.InvalidationListener;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;

import javax.inject.Inject;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * ScsPane
 */
public class ScsPane extends AbstractPreferencesSubPane {
    private final GeniUserProvider geniUserProvider;
    private final AuthorityList authorityList;
    private final JFedGuiPreferences jFedPreferences;
    @FXML protected Label prefSCSLabel;
    @FXML protected RadioButton autoScsRadioButton;
    @FXML protected RadioButton authScsRadioButton;
    @FXML protected RadioButton urlScsRadioButton;
    @FXML protected AuthorityChooser scsAuthChooser;
    @FXML protected Label scsUrlLabel;
    @FXML protected Label scsUrlChooseLabel;
    @FXML protected TextField scsUrlField;

    @FXML
    private BorderPane root;

    @Override
    public Node getRoot() {
        return root;
    }

    @Inject
    ScsPane(GeniUserProvider geniUserProvider, AuthorityList authorityList, JFedGuiPreferences jFedPreferences) {
        super("SCS", true);
        this.geniUserProvider = geniUserProvider;
        this.authorityList = authorityList;
        this.jFedPreferences = jFedPreferences;
    }

    @FXML
    private void initialize() {

        scsAuthChooser.showEditListButtonProperty().setValue(false);

        scsAuthChooser.managedProperty().bind(scsAuthChooser.visibleProperty());
        scsUrlLabel.managedProperty().bind(scsUrlLabel.visibleProperty());
        scsUrlChooseLabel.managedProperty().bind(scsUrlChooseLabel.visibleProperty());
        scsUrlField.managedProperty().bind(scsUrlField.visibleProperty());

        scsUrlChooseLabel.visibleProperty().bind(scsUrlField.visibleProperty());

        scsAuthChooser.visibleProperty().bind(authScsRadioButton.selectedProperty());
        scsUrlField.visibleProperty().bind(urlScsRadioButton.selectedProperty());
        scsUrlLabel.visibleProperty().bind(autoScsRadioButton.selectedProperty().or(authScsRadioButton.selectedProperty()));

        scsAuthChooser.setAuthorityFilter(ai -> ai.getSfaAuthority().getUrl(ServerType.GeniServerRole.SCS, 1) != null);

        InvalidationListener updateUrl = observable -> {
            if (authScsRadioButton.isSelected()) {
                AuthorityInfo newSelectedAuthority = scsAuthChooser.getSelectedAuthority();
                if (newSelectedAuthority == null) {
                    scsUrlLabel.setText("Error: no SCS selected.");
                    return;
                }
                URL scsUrl = newSelectedAuthority.getSfaAuthority().getUrl(ServerType.GeniServerRole.SCS, 1);
                if (scsUrl == null) {
                    scsUrlLabel.setText("Error: selected authority has no SCS URL");
                    return;
                }
                scsUrlLabel.setText("URL: "+scsUrl);

                //for convenience, so that something is added to the field
                if (scsUrlField.getText().trim().isEmpty())
                    scsUrlField.setText(scsUrl+"");
            }
        };

        scsAuthChooser.selectedAuthorityProperty().addListener(updateUrl);
        authScsRadioButton.selectedProperty().addListener(updateUrl);


        autoScsRadioButton.selectedProperty().addListener((observableValue, aBoolean, newIsSelected) -> {
            if (newIsSelected) {
                GeniUrn scsUrn = this.geniUserProvider.getLoggedInGeniUser().getUserAuthority().getDefaultScsUrn();
                SfaAuthority scs = this.authorityList.getAuthorityListModel().getFromAnyUrnExact(scsUrn+"");
                if (scs == null)
                    scsUrlLabel.setText("Error: user authority has an invalid SCS URN: "+scsUrn);
                else {
                    URL scsUrl = scs.getUrl(ServerType.GeniServerRole.SCS, 1);
                    if (scsUrl == null)
                        scsUrlLabel.setText("Error: user authority has no SCS URL");
                    else {
                        scsUrlLabel.setText("URL: "+scsUrl);

                        //for convenience, so that something is added to the field
                        if (scsUrlField.getText().trim().isEmpty())
                            scsUrlField.setText(scsUrl+"");
                    }
                }
            }
        });
        urlScsRadioButton.selectedProperty().addListener((observableValue, aBoolean, newIsSelected) -> {
            if (newIsSelected && scsUrlField.getText().trim().isEmpty()) {
                //scsUrlField.setText(scsUrl+"");
                scsUrlField.setText("http://MYSCS:8081/geni/xmlrpc");
            }
        });

        //start with auto selected. Then update from selection
        autoScsRadioButton.setSelected(true);

        URL scsUrl = jFedPreferences.getScsUrl();
        GeniUrn scsUrn = jFedPreferences.getScsUrn();
        if (scsUrn != null) {
            authScsRadioButton.setSelected(true);
            SfaAuthority selectedAuth = jFedPreferences.getScsAuthority(authorityList.getAuthorityListModel());
            if (selectedAuth != null)
                scsAuthChooser.select(selectedAuth);
        } else {
            if (scsUrl != null) {
                scsUrlField.setText(scsUrl+"");
                urlScsRadioButton.setSelected(true);
            } else {
                //trigger listener
                autoScsRadioButton.setSelected(false);
                autoScsRadioButton.setSelected(true);
            }
        }
    }
    @Override
    public boolean check() {
        if (urlScsRadioButton.isSelected()) {
            String urlTxt = scsUrlField.getText().trim();
            try {
                new URL(urlTxt);
            } catch (MalformedURLException e) {
                JFDialogs.create()
                        .owner(this)
                        .message("The provided SCS URL is not a valid URL")
                        .showError();
                return false;
            }
        }

        return true;
    }
    @Override
    public boolean save() {
        if (autoScsRadioButton.isSelected())
            jFedPreferences.setScs((URL) null);
        if (authScsRadioButton.isSelected()) {
            SfaAuthority auth = scsAuthChooser.getSelectedAuthority().getSfaAuthority();
            jFedPreferences.setScs(auth);
        }
        if (urlScsRadioButton.isSelected()) {
            String urlTxt = scsUrlField.getText().trim();
            try {
                URL u = new URL(urlTxt);
                jFedPreferences.setScs(u);
            } catch (MalformedURLException e) {
                JFDialogs.create()
                        .owner(this)
                        .message("The provided SCS URL is not a valid URL")
                        .showError();
                return false;
            }
        }


        return true;
    }
}
