package be.iminds.ilabt.jfed.experimenter_gui.slice.errors;

import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetails;
import be.iminds.ilabt.jfed.experimenter_gui.call_gui.TasksWindow;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.TaskExecutionFinishedListener;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.stitcher.ParallelStitcher;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsRef;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.ErrorDetails;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.application.Platform;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.ListChangeListener;
import javafx.event.EventHandler;
import javafx.scene.Cursor;
import javafx.scene.Group;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.input.MouseEvent;

import javax.inject.Inject;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * User: twalcari
 * Date: 7/2/2014
 * Time: 15:49
 */
public class ErrorsView extends ListView<ErrorDetails> implements TaskExecutionFinishedListener {

    private static final Set<String> IGNORABLE_GENI_RESPONSE_VALUES;

    private static final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    static {
        IGNORABLE_GENI_RESPONSE_VALUES = new HashSet<>();
        IGNORABLE_GENI_RESPONSE_VALUES.add("");
        IGNORABLE_GENI_RESPONSE_VALUES.add("0");
    }

    private final BooleanProperty newItemsAvailable = new SimpleBooleanProperty(false);

    {
        getItems().addListener((ListChangeListener<ErrorDetails>) change -> {
            while (change.next()) {
                if (change.wasAdded()) {
                    newItemsAvailable.set(true);
                }
            }
        });

        setCellFactory(errorDetailsListView -> new ErrorDetailsListCell());

        setPrefHeight(100);
    }

    private final TasksWindow tasksWindow;
    private final ApiCallDetailsCache apiCallDetailsCache;


    @Inject
    public ErrorsView(TasksWindow tasksWindow, ApiCallDetailsCache apiCallDetailsCache) {
        this.tasksWindow = tasksWindow;
        this.apiCallDetailsCache = apiCallDetailsCache;
    }

    public boolean getNewItemsAvailable() {
        return newItemsAvailable.get();
    }

    public BooleanProperty newItemsAvailableProperty() {
        return newItemsAvailable;
    }

    public void setNewItemsAvailable(boolean newItemsAvailable) {
        this.newItemsAvailable.set(newItemsAvailable);
    }

    @SuppressWarnings("ThrowableResultOfMethodCallIgnored")
    @Override
    public void onTaskExecutionFinished(Task task, TaskExecution taskExecution) {
        //TODO extract sliverUrn and componentUrn from tasks
        GeniUrn sliverUrn = null;
        GeniUrn componentUrn = null;

        boolean errorFoundInCall = false;

        for (ApiCallDetailsRef apiCallDetailsRef : taskExecution.getApiCallHistory()) {
            final SerializableApiCallDetails details = apiCallDetailsCache.getApiCallDetails(apiCallDetailsRef);

            if (details.getGeniResponseCodeIsSuccess() != null
                    && !details.getGeniResponseCodeIsSuccess()
                    && details.getGeniResponseOutput() != null) {

                StringBuilder message = new StringBuilder();
                message.append(task.getName()).append(": ");

                if (details.getXmlRpcGeniResponseValue() != null &&
                        !IGNORABLE_GENI_RESPONSE_VALUES.contains(details.getXmlRpcGeniResponseValue())) {
                    message.append(details.getXmlRpcGeniResponseValue());
                    message.append(" --- ");
                }
                message.append(details.getGeniResponseOutput());


                Platform.runLater(() -> getItems().add(new ApiCallErrorDetails(
                        message.toString(), sliverUrn, componentUrn, taskExecution)));
                errorFoundInCall = true;
            }
        }

        if (task instanceof ParallelStitcher.GetStatusUntilReadyOrFailTask) {
            ParallelStitcher.GetStatusUntilReadyOrFailTask getStatusUntilReadyOrFailTask =
                    (ParallelStitcher.GetStatusUntilReadyOrFailTask) task;
            final StatusDetails statusDetails = getStatusUntilReadyOrFailTask.getStatusDetails();

            if(!statusDetails.getAllErrors().isEmpty()){
                Platform.runLater(()-> getItems().addAll(statusDetails.getAllErrors()));
            }
        }

        if (task instanceof HighLevelController.GetStatusUntilReadyOrFailTask) {
            HighLevelController.GetStatusUntilReadyOrFailTask getStatusUntilReadyOrFailTask =
                    (HighLevelController.GetStatusUntilReadyOrFailTask) task;
            final StatusDetails statusDetails = getStatusUntilReadyOrFailTask.getStatusDetails();

            if(!statusDetails.getAllErrors().isEmpty()){
                Platform.runLater(()-> getItems().addAll(statusDetails.getAllErrors()));
            }
        }


        if (!errorFoundInCall && taskExecution.getException() != null) {
            Platform.runLater(() -> getItems().add(new ApiCallErrorDetails(task.getName() + ": " + taskExecution.getException(), taskExecution)));
        }


    }

    private class ErrorDetailsListCell extends ListCell<ErrorDetails> implements EventHandler<MouseEvent> {

        private final Label label = new Label();
        private final Group group = new Group();

        {
            label.maxWidthProperty().bind(ErrorsView.this.widthProperty());
            label.prefWidthProperty().bind(ErrorsView.this.widthProperty().subtract(3));
            label.setWrapText(true);
            label.setOnMouseClicked(this);


            group.getChildren().add(label);
            setGraphic(group);
        }

        @Override
        protected void updateItem(ErrorDetails errorDetails, boolean b) {
            super.updateItem(errorDetails, b);
            if (errorDetails != null) {
                StringBuilder sb = new StringBuilder();
                sb.append(DATE_FORMAT.format(new Date(errorDetails.getTimestamp())));
                sb.append(" ");
                if (errorDetails.getSliverUrn() != null) {
                    sb.append("[");
                    sb.append(errorDetails.getSliverUrn().getEncodedResourceName());
                    sb.append("] ");
                }

                if (errorDetails.getComponentUrn() != null) {
                    sb.append("{");
                    sb.append(errorDetails.getComponentUrn().getEncodedResourceName());
                    sb.append("} ");
                }

                sb.append(errorDetails.getError());


                label.setText(sb.toString());

                //determine if this call is clickable
                if (errorDetails instanceof ApiCallErrorDetails) {
                    setCursor(Cursor.HAND);
                } else {
                    setCursor(null);
                }


            } else {
                label.setText("");
                setCursor(null);
            }
        }

        @Override
        public void handle(MouseEvent mouseEvent) {
            if (this.getItem() instanceof ApiCallErrorDetails) {
                tasksWindow.showTaskExecution(((ApiCallErrorDetails) this.getItem()).getTaskExecution());
            }
        }
    }

//    public TaskExceptionListener createTaskExceptionListener(GeniUrn sliverUrn, GeniUrn componentUrn) {
//        return new TaskExceptionListener(sliverUrn, componentUrn, this, apiCallDetailsCache);
//    }
//
//    public void addExceptionListenerToTask(Task task, GeniUrn sliverUrn, GeniUrn componentUrn) {
//        //prevent multiple exception-listeners one one task
//        for (TaskFinishedCallback taskFinishedCallback : task.getCallbacks()) {
//            if (taskFinishedCallback instanceof TaskExceptionListener)
//                return; //there already is an exceptionlistener
//        }
//
//        task.addCallback(createTaskExceptionListener(sliverUrn, componentUrn));
//    }
//
//    public void addExceptionListenerToTask(Task task, GeniUrn sliverUrn) {
//        addExceptionListenerToTask(task, sliverUrn, null);
//    }
//
//    public void addExceptionListenerToTask(Task task) {
//        addExceptionListenerToTask(task, null, null);
//    }

}
