package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 14:24
 */
public abstract class SingleCallExperimentTask extends ExperimentTask {
    private static final Logger LOG = LoggerFactory.getLogger(SingleCallExperimentTask.class);

    protected final HighLevelController hlc;
    private final ObjectProperty<TaskExecution> taskExecution = new SimpleObjectProperty<>();

    private boolean finished = false;

    SingleCallExperimentTask(Experiment experiment, HighLevelController hlc) {
        super(experiment);
        this.hlc = hlc;
    }

    @Override
    protected final TaskStatusIndicator.Status executeTask() {
        LOG.trace("Starting execution of {} for {}", getClass().getSimpleName(), experiment.getName());

        onTaskStarting();

        Task task = createTask();

        TaskExecution taskExecution = hlc.submitTask(task);
        this.taskExecution.setValue(taskExecution);

        while (!taskExecution.isCompleted()) {
            try {
                Thread.sleep(250);
            } catch (InterruptedException ignored) {
            }
        }

        finished = true;

        registerFinishedTaskExecution(task, taskExecution);
        onTaskFinished(taskExecution);

        LOG.trace("Finished execution of {} for {}", getClass().getSimpleName(), experiment.getName());

        switch (taskExecution.getState()) {
            case SUCCESS:
                return TaskStatusIndicator.Status.SUCCESS;
            case FAILED:
            case CANCELLED:
                return TaskStatusIndicator.Status.FAILED;
            default:
                throw new IllegalStateException("Did not expected state " + taskExecution.getState().name()
                        + " for finished task " + task.toString());
        }
    }

    public void onTaskStarting() {

    }

    public boolean isFinished() {
        return finished;
    }

    public void onTaskFinished(TaskExecution taskExecution) {
    }

    public abstract Task createTask();

    public TaskExecution getTaskExecution() {
        return taskExecution.get();
    }

    public ObjectProperty<TaskExecution> taskExecutionProperty() {
        return taskExecution;
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }
}
