package be.iminds.ilabt.jfed.ssh_terminal_tool.standalone_test_app;

import be.iminds.ilabt.jfed.experimenter_gui.preferences.PreferencesDialogFactory;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.SshKeyInfo;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.ssh_terminal_tool.ExternalSshTerminal;
import be.iminds.ilabt.jfed.ssh_terminal_tool.ssh_key_info.UserSshKeyInfo;
import be.iminds.ilabt.jfed.util.KeyUtil;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;

/**
 * SshTerminalTestAppPanel
 */
public class SshTerminalTestAppPanel extends BorderPane {
    private static final Logger LOG = LoggerFactory.getLogger(SshTerminalTestAppPanel.class);
    private static final String SSH_TERMINAL_TEST_APP_GUI_FXML = "SshTerminalTestApp.fxml";

    private final GeniUserProvider geniUserProvider;
    private final ExternalSshTerminal externalSshTerminal;
    private final PreferencesDialogFactory preferencesDialogFactory;
    private final JFedGuiPreferences jFedPreferences;

    @FXML
    private TextArea sshKeyLabel;
    @FXML
    private Label proxyEnabledLabel;
    @FXML
    private Label sshAgentEnabledLabel;
    @FXML
    private TextField testHostLabel;
    @FXML
    private TextField testPortLabel;
    @FXML
    private TextField usernameLabel;

    @Inject
    SshTerminalTestAppPanel(GeniUserProvider geniUserProvider,
                            ExternalSshTerminal externalSshTerminal,
                            PreferencesDialogFactory preferencesDialogFactory,
                            JFedGuiPreferences jFedPreferences) {
        this.geniUserProvider = geniUserProvider;
        this.externalSshTerminal = externalSshTerminal;
        this.preferencesDialogFactory = preferencesDialogFactory;
        this.jFedPreferences = jFedPreferences;
    }


    @FXML
    private void initialize() {
        LOG.debug("Ssh Terminal Test App initialize");

        if (geniUserProvider.isUserLoggedIn())
            usernameLabel.setText(geniUserProvider.getLoggedInGeniUser().getUserUrn().getEncodedResourceName());

        updateLabels();
    }

    public void updateLabels() {
        SshKeyInfo sshKeyInfo;
        if (geniUserProvider.isUserLoggedIn()) {
            GeniUser geniUser = geniUserProvider.getLoggedInGeniUser();

            SshKeyInfo overrideSshKeyInfo = jFedPreferences.getOverriddenSshKeyInfo();
            if (overrideSshKeyInfo != null) {
                LOG.info("ExternalSshTerminal: The preferences are overriding the SSH key used (" + overrideSshKeyInfo.getClass().getName() + ").");
                sshKeyInfo = overrideSshKeyInfo;
            } else {
                LOG.info("ExternalSshTerminal: The user's SSH key will be used.");
                UserSshKeyInfo userSshKeyInfo = new UserSshKeyInfo(geniUser);
                sshKeyInfo = userSshKeyInfo;
            }
            String sshKey = KeyUtil.publicKeyToOpenSshAuthorizedKeysFormat(sshKeyInfo.getPublicKey());
            sshKeyLabel.setText(sshKey);
        } else {
            sshKeyLabel.setText("No user logged in");
        }


        proxyEnabledLabel.setText(jFedPreferences.getString(GuiPreferenceKey.PREF_SSHPROXY_USE_FOR_SSH));

        sshAgentEnabledLabel.setText(
                jFedPreferences.getBoolean(GuiPreferenceKey.PREF_SSHAGENT_USE) ? "true" : "false");

//        testHostLabel.setText();
//        testPortLabel.setText();
    }

    @FXML
    public void editPreferences(ActionEvent actionEvent) {
        LOG.debug("Ssh Terminal Test App editPreferences");

        preferencesDialogFactory.showDialog();

        updateLabels();
    }

    @FXML
    public void openTerminal(ActionEvent actionEvent) {
        LOG.debug("Ssh Terminal Test App openTerminal");
        assert geniUserProvider.isUserLoggedIn();

        GeniUser geniUser = geniUserProvider.getLoggedInGeniUser();
        UserSshKeyInfo userSshKeyInfo = new UserSshKeyInfo(geniUser);

        String targetServerUserName = usernameLabel.getText();
        String hostName = testHostLabel.getText();
        int port = Integer.parseInt(testPortLabel.getText());


        externalSshTerminal.launch(
                targetServerUserName,
                hostName,
                port,

                //info on the authority proxy. Note that this might not be used, if the preferences specify another proxy!
                geniUser.getUserAuthority().getProxies(),
                geniUser/* user */,
                userSshKeyInfo/* this is always the SSH key from log file. Even if other key is used, this is the info needed. The externalSshterminal will get key from preferences if needed. */
        );
    }

}
