package be.iminds.ilabt.jfed.highlevel.history;

import be.iminds.ilabt.jfed.highlevel.stitcher.StitchingReport;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * StitchingReportHistory
 * <p/>
 * Stores all StitchingReports seen in this session.
 * They are kept so they can be stored in a bugreport..
 */
@Singleton
public class StitchingReportHistory {
    private final LinkedList<StitchingReport> reports = new LinkedList<>();

    @Inject
    public StitchingReportHistory() {

    }

    public List<StitchingReport> getStitchingReports() {
        return Collections.unmodifiableList(reports);
    }

    public int size() {
        return reports.size();
    }

    public void addStitchingReport(StitchingReport sr) {
        //for memory consumption reasons, store only the last 2 reports
        synchronized (this) {
            reports.add(sr);

            while (reports.size() > 2)
                reports.removeFirst();
        }
    }
}
