package be.iminds.ilabt.jfed.highlevel.stitcher;

import be.iminds.ilabt.jfed.lowlevel.CredentialException;
import be.iminds.ilabt.jfed.lowlevel.SfaCredential;
import be.iminds.ilabt.jfed.lowlevel.TestCertHelper;
import be.iminds.ilabt.jfed.lowlevel.connection.MockSfaConnection;
import be.iminds.ilabt.jfed.util.GeniUrn;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Date;

/**
 * TestStitchingAMv2
 */
public class TestStitchingSAServer extends MockSfaConnection.AbstractMockXmlRpcReply {
    private final static Logger LOG = LoggerFactory.getLogger(TestStitchingSAServer.class);

    private static int globalIndex = 0;

    protected final String serverUrl;

    protected final String saUrnString;
    protected final GeniUrn saUrn;
    protected final String saHostname;

    protected final TestCertHelper testCertHelper;

    public TestStitchingSAServer(String serverUrl, String saUrnString, String saHostname, TestCertHelper testCertHelper) {
        this.serverUrl = serverUrl;
        this.saHostname = saHostname;
        this.saUrnString = saUrnString;
        try {
            this.saUrn = new GeniUrn(saUrnString);
        } catch (GeniUrn.GeniUrnParseException e) {
            throw new RuntimeException(e);
        }

        this.testCertHelper = testCertHelper;
    }

    @Override
    public MockSfaConnection.MockXmlRpcAnswer matches(MockSfaConnection.MockXmlRpcRequestInfo info) {
        if (!info.serverUrl.equals(serverUrl))
            return null;

        if (info.method.equals("GetCredential") && info.params.size() >= 0) {
            String targetUrn = TestCertHelper.userUrn;
            X509Certificate ownerCert = testCertHelper.saCert;
            X509Certificate targetCert = testCertHelper.userCert;
            X509Certificate signerCert = testCertHelper.authority1Cert;
            PrivateKey signerPrivateKey = testCertHelper.authority1Keys.getSshPrivateKey();
            Date expireDate = new Date(System.currentTimeMillis() + (24*60*60));
            String privilegeName = "*";
            boolean canDelegate = true;
            String credentialjFedName = "Test Credential for Test User";
            SfaCredential cred = null;
            try {
                cred = SfaCredential.create(
                        "privilege",
                        saUrnString/*ownerUrn*/, targetUrn,
                        ownerCert, targetCert,
                        signerCert, signerPrivateKey,
                        expireDate, privilegeName, canDelegate,
                        credentialjFedName);
            } catch (CredentialException e) {
                LOG.debug("Error creating test credential", e);
                throw new RuntimeException(e);
            }
            String credentialXml = cred.getCredentialXml();

            return new MockSfaConnection.MockXmlRpcAnswer(TestStitchingXmlRpcHelper.wrapSuccessXmlRpcSAReply(TestStitchingXmlRpcHelper.encodeXmlRpcString(credentialXml)));
        }
        return null;
    }
}
