package be.iminds.ilabt.jfed;

import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.JFedAuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.UtahClearingHouseAuthorityList;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.name.Names;
import com.google.inject.util.Providers;

import javax.inject.Singleton;
import java.io.File;

/**
 * User: twalcari
 * Date: 1/13/2015
 * Time: 11:40
 */
public class BasicAuthoritiesModule extends AbstractModule {
    private File authoritiesFile = JFedAuthorityList.getDefaultAuthoritiesFile();
    private boolean fetchFromGeniClearingHouse = false;

    public BasicAuthoritiesModule() {
    }

    @Override
    protected void configure() {
        bind(File.class).annotatedWith(Names.named("authoritiesFile")).toProvider(Providers.of(authoritiesFile));
    }

    @Provides
    @Singleton
    private AuthorityFinder provideAuthorityFinder(AuthorityListModel authorityListModel, JFedPreferences jFedPreferences) {
        return new AuthorityFinder(authorityListModel, jFedPreferences);
    }

    @Provides
    @Singleton
    private AuthorityListModel provideAuthorityListModel(JFedAuthorityList jFedAuthorityList, JFedTrustStore trustStore) {
        AuthorityListModel authorityListModel = jFedAuthorityList.get();
        if (fetchFromGeniClearingHouse) {
            UtahClearingHouseAuthorityList.load(authorityListModel);
            UtahClearingHouseAuthorityList.addToTrustStore(trustStore); //these can be different certificates than the ones in authorityListModel (separate download)
        }
        trustStore.addAuthorityCerts(authorityListModel);
        return authorityListModel;
    }


    public void setAuthoritiesFile(File authoritiesFile) {
        this.authoritiesFile = authoritiesFile;
    }

    public void setFetchFromGeniClearingHouse(boolean fetchFromGeniClearingHouse) {
        this.fetchFromGeniClearingHouse = fetchFromGeniClearingHouse;
    }
}
