package be.iminds.ilabt.jfed.lowlevel.api_wrapper;

import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.AnyCredential;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.SfaCredential;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * ApiWrapper
 */
public abstract class ApiWrapper {
    private final static org.slf4j.Logger parentLOG = LoggerFactory.getLogger(ApiWrapper.class);

    protected final Logger logger;
    protected final GeniUserProvider geniUserProvider;
    protected final JFedConnectionProvider connectionProvider;
    protected final JFedPreferences jFedPreferences;

    protected ApiWrapper(Logger logger, GeniUserProvider geniUserProvider, JFedConnectionProvider connectionProvider, JFedPreferences jFedPreferences) {
        assert logger != null;
        assert geniUserProvider != null;
        assert connectionProvider != null;
        assert jFedPreferences != null;

        this.logger = logger;
        this.geniUserProvider = geniUserProvider;
        this.connectionProvider = connectionProvider;
        this.jFedPreferences = jFedPreferences;
    }

    /**
     * returns only the SfaCredentials in the list, discarding the ABAC and other credentials.
     * */
    public List<AnyCredential> filterCredentials(List<AnyCredential> origCredentials) {
        if (origCredentials == null)
            return null;
        List<AnyCredential> res = new ArrayList<AnyCredential>();
        for (AnyCredential cred : origCredentials) {
            if (cred instanceof SfaCredential) {
                res.add(cred);
            }
        }
        if (res.isEmpty() && !origCredentials.isEmpty()) {
            parentLOG.warn("Unknown what is a good credential to use. Will just use first one received (type=" + origCredentials.get(0).getClass().getName() + ").");
            res.add(origCredentials.get(0));
        }
        return res;
    }
}
