package be.iminds.ilabt.jfed.util;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.Deque;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Vector;

/**
 * JSonHelper
 */
public class JSonHelper {
    private static interface JsonHelper {
        public Object getSourceObject();
        public void setTranslation(Object o);
    }
    private static class PrimitiveHelper implements JsonHelper {
        Object source;
        Object translation;

        public Object getSourceObject() { return source; }
        public void setTranslation(Object o) { translation = o; }
    }
    private static class VectorHelper implements JsonHelper {
        Vector target;
        int index;
        Object item;

        public Object getSourceObject() { return item; }
        public void setTranslation(Object o) { target.set(index, o); }
    }
    private static class HashtableHelper implements JsonHelper {
        Hashtable target;
        Object key;
        Object value;

        public Object getSourceObject() { return value; }
        public void setTranslation(Object o) { target.put(key, o); }
    }

    /**
     * converts into JSon object using Vector and Hashtable
     * */
    public static Object jsonStringToXmlRpcLikeObject(String jsonString) {
        if (jsonString == null) return null;
        if (jsonString.trim().isEmpty()) return null;
        Deque<JsonHelper> objectsToProcess = new LinkedList<JsonHelper>();
        PrimitiveHelper base = new PrimitiveHelper();
        base.source = new JSONTokener(jsonString).nextValue();
        objectsToProcess.add(base);
        while (!objectsToProcess.isEmpty()) {
            JsonHelper jsonHelper = objectsToProcess.pollFirst();

            boolean handled = false;
            if (jsonHelper.getSourceObject() instanceof JSONObject) {
                handled = true;
                JSONObject o = (JSONObject) jsonHelper.getSourceObject();
                Hashtable ht = new Hashtable();
                for (Object key : o.keySet()) {
                    HashtableHelper htHelp = new HashtableHelper();
                    htHelp.target = ht;
                    htHelp.key = key; //assumes key is primitive type and not something else!
                    htHelp.value = o.get((String)key); //this actually seems to assume keys can only be String  (which is fine)
                    objectsToProcess.add(htHelp);
                }
                jsonHelper.setTranslation(ht);
            }
            if (jsonHelper.getSourceObject() instanceof JSONArray) {
                handled = true;
                JSONArray a = (JSONArray) jsonHelper.getSourceObject();
                Vector v = new Vector(a.length());
                for (int i = 0; i < a.length(); i++) {
                    v.add(null); //init v
                    VectorHelper vh = new VectorHelper();
                    vh.target = v;
                    vh.index = i;
                    vh.item = a.get(i);
                    objectsToProcess.add(vh);
                }
                jsonHelper.setTranslation(v);
            }
            if (!handled) {
                jsonHelper.setTranslation(jsonHelper.getSourceObject());
            }
        }

        return base.translation;
    }

    public static String jsonObjectToString(Object o) {
        return XmlRpcPrintUtil.xmlRpcObjectToString(o);
    }
}
