package be.iminds.ilabt.jfed.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * LocalStringUtils
 */
public class TextUtil {
    private static final Logger LOG = LoggerFactory.getLogger(TextUtil.class);

    public static String printArray(String[] arr) {
        if (arr == null) return "null";
        String res = "[";
        boolean first = true;
        for (String a : arr) {
            if (!first)
                res += ", ";
            res += a;
            first = false;
        }
        res += "]";
        return res;
    }

    /**
     * @return the input string, with newlines inserted so that no line is longer than 'maxLineLen'
     */
    public static String wrap(String s, int maxLineLen) {
        if (s.length() < maxLineLen)
            return s;
        String updated = "";
        int linecount = 0;
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            if (c == '\n') linecount = 0;
            if (linecount >= maxLineLen) {
                updated += '\n';
                linecount = 0;
            }
            if (c != '\n') linecount++;
            updated += c;
        }

        return updated;
    }

    /**
     * @return the input string, with newlines inserted so that no line is longer than 'maxLineLen'
     */
    public static String abbreviate(String s, int maxLineLen) {
        if (s.length() < maxLineLen)
            return s;
        return s.substring(0, maxLineLen) + "...";
    }

    public static List<String> getLines(String in) {
        List<String> res = new ArrayList<String>();

        for (String part : in.split("\n"))
            res.add(part);

        return res;
    }

    public static String mkString(Collection<?> c, String sep) {
        String res = "";
        for (Object i : c) {
            if (!res.isEmpty()) res += sep;
            res += i.toString();
        }
        return res;
    }

    /**
     * @returns the boolean represented by this string or boolean, or null if not a boolean or string representation of a boolean
     */
    public static Boolean objectToBoolean(Object val) {
        if (val == null) return null;
        if (val instanceof String)
            return stringToBoolean((String) val);
        if (val instanceof Boolean)
            return (Boolean) val;
        return null;
    }

    /**
     * Create a String or null from an object.
     *
     * @param val the possible string. May be null.
     * @param makeString if true, will make a string from val if it is non null, even if it is not of the String class. If false, return null if val is not a String.
     * @return
     */
    public static String objectToStringOrNull(@Nullable Object val, boolean makeString) {
        if (val == null) return null;
        if (val instanceof String)
            return (String) val;
        if (makeString)
            return val+"";
        else
            return null;
    }

    /**
     * @returns the boolean represented by this string, or null if not a boolean
     */
    public static Boolean stringToBoolean(String val) {
        if (val.equalsIgnoreCase("true"))
            return true;
        if (val.equalsIgnoreCase("yes"))
            return true;
        if (val.equalsIgnoreCase("yep"))   //seen in <link_attribute xmlns="http://www.protogeni.net/resources/rspec/ext/emulab/1" key="nomac_learning" value="yep" />
            return true;
        if (val.equalsIgnoreCase("1"))
            return true;
        if (val.equalsIgnoreCase("false"))
            return false;
        if (val.equalsIgnoreCase("no"))
            return false;
        if (val.equalsIgnoreCase("nope"))
            return false;
        if (val.equalsIgnoreCase("0"))
            return false;
        return null;
    }

    public static String exceptionToString(Throwable exception) {
        assert (exception != null);
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        exception.printStackTrace(pw);
        String stackTrace = sw.toString();
        return stackTrace;
    }

    /**
     * indent the lines in the provided string.
     * This is done by adding <count> spaces before each line.
     *
     * TODO: this implementation can be improved by using something other than the BufferedReader
     *
     * @param count nr of lines to indent each line.
     */
    public static String indent(int count, String text) {
        if (text == null)
            return null;
        String spaces = "";
        for (int i = 0; i < count; i++)
            spaces += " ";
        char lastChar = text.isEmpty() ? ' ' : text.charAt(text.length()-1);
        boolean endsWithNewLine = lastChar == '\n' || lastChar == '\r';
        String result = "";
        BufferedReader br = null;
        try {
            br = new BufferedReader(new StringReader(text));
            String line = br.readLine();
            while (line != null) {
                result += spaces + line + "\n"; //note: this replaces any line ending with \n
                line = br.readLine();
            }
        } catch (IOException e) {
            LOG.error("Got an IOException where there should never be one", e);
            throw new RuntimeException(e);
        } finally {
            if (br != null)
                try {
                    br.close();
                } catch (IOException e) {
                    LOG.error("Got an IOException where there should never be one", e);
                }
        }
        if (!endsWithNewLine && !result.isEmpty())
            return result.substring(0, result.length()-1);
        return result;
    }
}