package be.iminds.ilabt.jfed.rspec.model;

import org.testng.annotations.Test;

import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;

/**
 * BasicStringRspecTest
 */
public class BasicStringRspecTest {
    private static final String wall2Manifest =
            "<rspec xmlns=\"http://www.geni.net/resources/rspec/3\" xmlns:client=\"http://www.protogeni.net/resources/rspec/ext/client/1\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" type=\"manifest\" xsi:schemaLocation=\"http://www.geni.net/resources/rspec/3 http://www.geni.net/resources/rspec/3/manifest.xsd \" expires=\"2014-09-01T06:15:12Z\">\n  " +
                    "   <node client_id=\"PC\" component_manager_id=\"urn:publicid:IDN+wall2.ilabt.iminds.be+authority+cm\" exclusive=\"true\" component_id=\"urn:publicid:IDN+wall2.ilabt.iminds.be+node+n091-25\" sliver_id=\"urn:publicid:IDN+wall2.ilabt.iminds.be+sliver+28109\">\n" +
                    "      <sliver_type name=\"raw-pc\"/>\n" +
                    "      <rs:vnode xmlns:rs=\"http://www.protogeni.net/resources/rspec/ext/emulab/1\" name=\"n091-25\"/><host name=\"PC.sTKe1409551752.ch-geni-net.wall2.ilabt.iminds.be\"/>" +
                    "      <services><login authentication=\"ssh-keys\" hostname=\"n091-25.wall2.ilabt.iminds.be\" port=\"22\" username=\"wimvande\"/></services>" +
                    "   </node>\n" +
                    "</rspec>";

    @Test
    public void testWall2Manifest() {
        BasicStringRspec basicStringRspec = new BasicStringRspec(wall2Manifest);

        List<BasicStringRspec.BasicNodeInfo> basicNodeInfos = basicStringRspec.getBasicNodeInfo();
        assertEquals(basicNodeInfos.size(), 1);
        BasicStringRspec.BasicNodeInfo node0 = basicNodeInfos.get(0);
        assertNotNull(node0.getClientId());
        assertNotNull(node0.getComponentId());
        assertEquals(node0.getClientId(), "PC");
        assertEquals(node0.getComponentId(), "urn:publicid:IDN+wall2.ilabt.iminds.be+node+n091-25");
        assertEquals(node0.getComponentManagerId(), "urn:publicid:IDN+wall2.ilabt.iminds.be+authority+cm");
        assertEquals(node0.getUniqueId(), "PC@urn:publicid:IDN+wall2.ilabt.iminds.be+authority+cm");

        List<BasicStringRspec.LoginService> loginServiceList = basicStringRspec.findNodeLoginInfoByUniqueId(
                node0.getUniqueId());
        assertEquals(loginServiceList.size(), 1);
        BasicStringRspec.LoginService login = loginServiceList.get(0);
        assertEquals(login.getAuthentication(), "ssh-keys");
        assertEquals(login.getHostname(), "n091-25.wall2.ilabt.iminds.be");
        assertEquals(login.getPort(), 22);
        assertEquals(login.getUsername(), "wimvande");
    }
}
