package be.iminds.ilabt.jfed.rspec.model.javafx_impl;

import be.iminds.ilabt.jfed.rspec.ext.jfedbonfire.BonfireNodeInfo;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.stream.Location;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLStreamException;
import java.util.ArrayList;
import java.util.List;

public class BonfireRspecNode extends FXRspecNode {
    private static final Logger LOG = LoggerFactory.getLogger(BonfireRspecNode.class);

    private final ObjectProperty<BonfireNodeInfo> bonfireNodeInfo = new SimpleObjectProperty<>();

    public BonfireRspecNode(FXModelRspec modelRspec, String id) {
        super(modelRspec, id);
    }

    public BonfireRspecNode(FXModelRspec newModelRspec, FXRspecNode rspecNode, InterfaceCopyMethod interfaceCopyMethod) {
        super(newModelRspec, rspecNode, interfaceCopyMethod);

        if (rspecNode instanceof BonfireRspecNode) {
            BonfireRspecNode bonfireRspecNode = (BonfireRspecNode) rspecNode;
            if (bonfireRspecNode.bonfireNodeInfo.get() != null)
                this.bonfireNodeInfo.set(new BonfireNodeInfo(bonfireRspecNode.bonfireNodeInfo.get()));
            else
                this.bonfireNodeInfo.set(null);
        }
    }

    public BonfireRspecNode(FXModelRspec newModelRspec, FXRspecNode node) {
        this(newModelRspec, node, InterfaceCopyMethod.NO_COPY);
    }

    //you can edit in place, or replace
    public BonfireNodeInfo getBonfireNodeInfo() {
        return bonfireNodeInfo.get();
    }

    public void setBonfireNodeInfo(BonfireNodeInfo value) {
        bonfireNodeInfo.set(value);
    }

    public ObjectProperty<BonfireNodeInfo> bonfireNodeInfoProperty() {
        return bonfireNodeInfo;
    }

    @Override
    @Nonnull
    public List<ExtraXml> getExtraXml() {
        List<ExtraXml> res = new ArrayList<>(super.getExtraXml());

        if (bonfireNodeInfo.get() != null) {
            res.add(new ExtraXml() {
                @Override
                public void writeEvents(XMLEventWriter w) throws XMLStreamException {
                    JAXBElement<BonfireNodeInfo> je = new JAXBElement<>(FXRspecFactory.Q_BONFIRE_NODE_INFO, BonfireNodeInfo.class, bonfireNodeInfo.get());

                    try {
                        JAXBContext jc = JAXBContext.newInstance(BonfireNodeInfo.class);
                        Marshaller marshaller = jc.createMarshaller();
                        marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);
                        marshaller.marshal(je, w);
                    } catch (JAXBException e) {
                        throw new RuntimeException("error writing BonfireNodeInfo", e);
                    }
                }

                @Override
                public QName getStartElementName() {
                    return FXRspecFactory.Q_BONFIRE_NODE_INFO;
                }

                @Override
                public Location getLocation() {
                    return null;
                }
            });
        }

        return res;
    }
}
