package be.iminds.ilabt.jfed.rspec.model;

import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import be.iminds.ilabt.jfed.rspec.parser.RspecCompare;
import be.iminds.ilabt.jfed.util.GeniUrn;

import javax.annotation.Nullable;
import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * ModelRspec
 */
public interface ModelRspec {
    ModelRspecType getModelRspecType();

    void setParseData(String schemaLocation, String defaultNamespace, Map<String, String> namespaces);

    @Nullable
    String getSchemaLocation();

    @Nullable
    String getOrigDefaultNamespace();

    Map<String, String> getNameSpaceInfo();

    void addNode(RspecNode node);
    List<? extends RspecNode> getNodes();

    void addLink(RspecLink link);
    List<? extends RspecLink> getLinks();

    void addOpenflowDataPath(OpenflowDataPath dp);
    List<? extends OpenflowDataPath> getOpenflowDataPaths();

    void addLease(Lease lease);
    List<? extends Lease> getLeases();

    void addChannel(Channel channel);
    List<? extends Channel> getChannels();

    @Nullable
    Channel getChannelByComponentId(GeniUrn componentId);

    @Nullable
    RspecNode getNodeByUniqueId(String id);
    @Nullable
    RspecNode getNodeByClientId(String id);
    @Nullable
    RspecNode getNodeByComponentId(String id);

    List<? extends RspecNode> getNodesByAuthority(SfaAuthority authority);

    @Nullable
    RspecLink getLinkByClientId(String id);
    @Nullable
    RspecLink getLinkByUniqueId(String id);

    /**
     * Deletes a node and all interfaces attached to it. If a link becomes surplus, it is removed also.
     *
     * @param node
     */
    void deleteNode(RspecNode node);

    /**
     * Removes the link from the Rspec
     *
     * @param link
     */
    void deleteLink(RspecLink link);


    boolean deleteChannel(Channel channel);

    List<GeniUrn> getAllComponentManagerUrns();

    List<UserSpec> getSshKeys();
    void setSshKeys(List<UserSpec> newKeys);

    String nextLinkName();

    String nextNodeName();

    String getType();

    String nextIfaceName(RspecNode node);

    @Nullable
    RspecInterface getInterfaceByUniqueId(String uniqueId);
    @Nullable
    RspecInterface getInterfaceByClientId(String clientId);

    List<ExtraXml> getExtraXml();
    void setExtraXml(List<ExtraXml> extraXml);

    /**
     * @return true if data was lost in creating this model from an xml string, false if not losing data. null otherwise
     * (including when this was not constructed from xml, or from manifest or advertisement rspec)
     */
    Boolean isLosingData();
    List<RspecCompare.Difference> getLosingDataDifferences();

    void setLosingData(Boolean losingData, List<RspecCompare.Difference> differences);

    /**
     * For the moment, this does nothing more than call toGeni3RequestRspec_internal and write Xml To string.
     * Later, this can be edited to handle special cases in the geni request rspec v3 generation.
     * <p/>
     * only the PLE case is handled now, but it works automatically, without a need to specify it.
     */
    String toGeni3Rspec(RequestRspecSpecialCases specialCase);

    String toGeni3Rspec();

    @Nullable
    Date getExpires() throws ParseException;
    void setExpires(Date expires);
    String getExpiresString();
    void setExpires(String expires);



    enum RequestRspecSpecialCases {NONE, PLE, NITOS}

    ModelRspec copy();
}
